/*
Tecnisa - Tecnologia em Informtica

Nome: APURAC_REFESERV

Descrio:
   Procedimento para apurao de faturamento de clientes em filiais
   que utilizam o faturamento de Refeicoes e Servicos.

Criao:
22/05/2001 - Disnei H. de Moura

@C:\Win_V4\SQL\END_CONV\ORACLE\6205_FAISTPOApurRefeServ.sql;
raise_application_error(-20501, '' , True);
*/

CREATE OR REPLACE PROCEDURE APURAC_REFESERV(
   P_CDFILIAPUR  IN FILIAL.CDFILIAL%TYPE,
   P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
   P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE,
   P_DATAINI     IN DATE,
   P_DATAFIM     IN DATE,
   P_NRAPURACFAT IN NOTAFISCAL.NRAPURACFAT%TYPE,
   P_NRLANCTONF  IN NOTAFISCAL.NRLANCTONF%TYPE) AS


   /* DECLARAO DO TIPO (RECORD) QUE CONTERA OS DADOS DO CURSOR */
   TYPE GDiretaRecType IS RECORD (
      CdFilial       FILIAL.CDFILIAL%TYPE,
      CdCliente      CLIENTE.CDCLIENTE%TYPE,
      DtFaturam      DATE,
      CdServFili     SERVICO.CDSERVFILI%TYPE,
      CdGrServFat    GRUPSERVFAT.CDGRSERVFAT%TYPE,
      CdProduto      PRODUTO.CDPRODUTO%TYPE,
      VrServico      NUMBER,
      QtEfetivos     NUMBER);

   /* DECLARAO DO TIPO DO CURSOR */
   TYPE GDiretaCurType IS REF CURSOR RETURN GDiretaRecType;

   /* VARIVEIS QUE CONTERO O CURSOR E O REGISTRO */
   Rec_GDireta    GDiretaRecType;
   Cur_GDireta    GDiretaCurType;

   /* VARIVEIS PARA CONTROLE DE PARAMETROS */
   stIdTipNFEmp   VARCHAR2(01);
   stIdMiniEfet   VARCHAR2(01);
   stIdQtdePrec   VARCHAR2(01);
   stIdUtEfePla   VARCHAR2(01);

   /* VARIVEIS PARA CONTROLE DE APURAO */
   QtdeMiniEfet   EFECLIFAT.QTCOMEFATU%TYPE;
   VrMiniEfet     EFECLIFAT.QTCOMEFATU%TYPE;
   V_CdServAjus   SERVICO.CDSERVFILI%TYPE;
   V_QtEfetServ   NUMBER;
   V_QtEfetGrup   NUMBER;
   V_TtEfetServ   NUMBER;
   V_VrServico    NUMBER;
   V_VrServFatu   NUMBER;
   V_VrPercCorr   NUMBER;
   V_Existe       NUMBER;

   /* VARIAVEIS PARA CONTROLE DE GRAVACAO DA NOTA FISCAL */
   V_CHAVECST     VARCHAR2(14);
   V_CSTTEMP      VARCHAR2(03);
   V_IDPRODSUBS   VARCHAR2(01);
   V_IDPRODANTE   VARCHAR2(01);

   V_IDENDCFOP      VARCHAR2(01);
   V_PAISORIG       VARCHAR2(04);
   V_ESTAORIG       VARCHAR2(02);
   V_MUNIORIG       VARCHAR2(04);
   V_ORIGDESTOP     VARCHAR2(01);
   V_CDORIGDESTNF   VARCHAR2(21);
   V_IDENDEMIS      VARCHAR2(01);
   V_PAISDEST       VARCHAR2(04);
   V_ESTADEST       VARCHAR2(02);
   V_MUNIDEST       VARCHAR2(04);
   V_CDEMPRESADEST  VARCHAR2(02);
   V_CDINSCESTADEST VARCHAR2(16);
   V_IDINTEREST     VARCHAR2(01);
   V_IDINTERNA      VARCHAR2(01);

   V_NrSeqItProd  NUMBER;
   V_CdEmpresa    FILIAL.CDEMPRESA%TYPE;
   V_CdInscMuni   FILIAL.CDINSCMUNI%TYPE;
   V_CdInscEsta   FILIAL.CDINSCESTA%TYPE;
   V_IdTpIJurClie CLIENTE.IDTPIJURCLIE%TYPE;
   V_NrInsJurClie CLIENTE.NRINSJURCLIE%TYPE;
   V_IdNovaNota   VARCHAR2(01);
   V_VrNotaFisc   NUMBER;
   V_PrxLancEstq  VARCHAR2(10);

   V_CDEMPRATAC  EMPRESA.CDEMPRESA%TYPE;
   V_CDINSCATAC  INSCRESTAD.CDINSCESTA%TYPE;
   V_INSCMUATAC  FILIAL.CDINSCMUNI%TYPE;
   V_CDEMPROPER  EMPRESA.CDEMPRESA%TYPE;
   V_SERIEATAC   SERIENF.CDSERIE%TYPE;
   V_CDSERIE     SERIENF.CDSERIE%TYPE;
   V_INSCMUOPER  FILIAL.CDINSCMUNI%TYPE;
   V_CDINSCOPER  INSCRESTAD.CDINSCESTA%TYPE;
   V_SERIEOPER   SERIENF.CDSERIE%TYPE;

   V_NRLANCTONF  VARCHAR2(20);
   V_NRLANCESTQ  LANCTOESTOQ.NRLANCESTQ%TYPE;

   V_CDPRGENESER PRODUTO.CDPRODUTO%TYPE;
   V_NMPRGENESER PRODUTO.NMPRODUTO%TYPE;

   V_CDPRGENEREF PRODUTO.CDPRODUTO%TYPE;
   V_NMPRGENEREF PRODUTO.NMPRODUTO%TYPE;

   V_VRPERCSERV  NUMBER;
   V_VRTOTASERV  NUMBER;
   V_VRLIQUFATU  NUMBER;
   V_TTITENSNF   NUMBER;
   V_CONTITEM    NUMBER;
   V_VRNFATACA   NUMBER;

   V_CDGRUPOFISC  VARCHAR2(03);
   
   V_CDTPOPERREF PARAMGERAL.CDTPOPERREF%TYPE;
   V_CDTPOPERPRESER PARAMGERAL.CDTPOPERPRESER%TYPE;

   /* PROCEDIMENTO QUE ABRE O CURSOS DE ACORDO COM ALGUNS PARAMETROS
    PARAMETROS:
    1. CURSOR A SER ABERTO
    2. INDICADOR DO CONTROLE DE MINIMO DE EFETIVOS (POR DIA OU POR PERIODO)
    3. INDICADOR DA UTILIZACAO DE EFETIVOS DO PLANEJAMENTO
    4. INDICADOR DA UTILIZACAO DE GRUPOS DE SERVICOS */
   PROCEDURE OpenGDiretaCursor (C_GDireta  IN OUT GDiretaCurType,
                                V_MinEfet  IN VARCHAR2,
                                V_UtPlanej IN VARCHAR2,
                                V_Grupo    IN VARCHAR2) IS
   BEGIN

      IF (V_MinEfet = '2') THEN
      /* Controla por Periodo */
         OPEN C_GDireta FOR
            SELECT CDFILIAL, CDCLIENTE, NULL DTFAT, CDSERVFILI, CDGRSERVFAT,
                   CDPRODUTO, VRSERVICO, SUM(QTEFETIVOS)
            FROM SUM_EFETIVOS
            WHERE DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM AND
                  ((IDFATUPLAN = 'F') OR (V_UtPlanej = 'S')) AND
                  CDFILIAL   = P_CDFILIAPUR AND
                  CDCLIENTE  = P_CDCLIENTE
            GROUP BY CDFILIAL, CDCLIENTE, CDSERVFILI, CDGRSERVFAT, CDPRODUTO,
                     VRSERVICO
            HAVING (SUM(QTEFETIVOS) <> 0) AND
                   (((CDGRSERVFAT IS NULL) AND (V_Grupo = 'N')) OR
                    ((CDGRSERVFAT IS NOT NULL) AND (V_Grupo = 'S')));
      END IF;

      IF (V_MinEfet = '1') THEN
      /* Controla por Dia */
         OPEN C_GDireta FOR
            SELECT CDFILIAL, CDCLIENTE, DTFATURAM, CDSERVFILI, CDGRSERVFAT,
                   CDPRODUTO, VRSERVICO, SUM(QTEFETIVOS)
            FROM SUM_EFETIVOS
            WHERE DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM AND
                  ((IDFATUPLAN = 'F') OR (V_UtPlanej = 'S')) AND
                  CDFILIAL   = P_CDFILIAPUR AND
                  CDCLIENTE  = P_CDCLIENTE
            GROUP BY CDFILIAL, CDCLIENTE, DTFATURAM, CDSERVFILI, CDGRSERVFAT,
                     CDPRODUTO, VRSERVICO
            HAVING (SUM(QTEFETIVOS) <> 0) AND
                   (((CDGRSERVFAT IS NULL) AND (V_Grupo = 'N')) OR
                    ((CDGRSERVFAT IS NOT NULL) AND (V_Grupo = 'S')));
      END IF;
   END OpenGDiretaCursor;

BEGIN
   V_CHAVECST     := P_CDFILIPROC || P_NRAPURACFAT;
   V_VRLIQUFATU   := 0;
   V_VRTOTASERV   := 0;
   V_TTITENSNF    := 10;
   
   SELECT TO_NUMBER(NVL(MAX(SUBSTR(LTRIM(PF.DSPARAMFATFI),50,5)),'0')), /*Percentual de Compras/Revenda*/
          MAX(SUBSTR(PF.DSPARAMFATFI,39,2)),  /*Empresa atacadista*/
          MAX(SUBSTR(PF.DSPARAMFATFI,41,2)),  /*Empresa operacional*/
          LTRIM(RTRIM(MAX(SUBSTR(PG.DSPARAMFATGE,105,10)))), /*Codigo da Produto para Refeio*/
          LTRIM(RTRIM(MAX(SUBSTR(PG.DSPARAMFATGE,115,10)))) /*Codigo da Produto para Servico*/
   INTO V_VRPERCSERV, V_CDEMPRATAC,  V_CDEMPROPER, V_CDPRGENESER, V_CDPRGENEREF
   FROM PARAMFILIAL PF, PARAMGERAL PG
   WHERE PF.CDFILIAL = P_CDFILIAPUR;


   V_VRPERCSERV := V_VRPERCSERV/1000;

   SELECT MAX(NMPRODUTO)
   INTO V_NMPRGENESER
   FROM PRODUTO
   WHERE CDPRODUTO  =  V_CDPRGENESER;

   SELECT MAX(NMPRODUTO)
   INTO V_NMPRGENEREF
   FROM PRODUTO
   WHERE CDPRODUTO  =  V_CDPRGENEREF;

   /* BUSCA AS INFORMAOES PARA CADASTRAMENTO DA NOTA FISCAL */
   SELECT MAX(CDEMPRESA), MAX(CDINSCMUNI), MAX(CDINSCESTA), MAX(CDSERIE)
   INTO V_CdEmpresa, V_CdInscMuni, V_CdInscEsta, V_CDSERIE
   FROM FILIAL
   WHERE CDFILIAL = P_CDFILIAPUR;

   SELECT MAX(CDINSCMUNI), MAX(CDINSCESTA), MAX(CDSERIE)
   INTO V_INSCMUATAC, V_CDINSCATAC, V_SERIEATAC
   FROM EMPRCNTRFIL
   WHERE CDEMPRESA = V_CDEMPRATAC   AND
         CDFILIAL  = P_CDFILIAPUR;

   SELECT MAX(CDINSCMUNI), MAX(CDINSCESTA), MAX(CDSERIE)
   INTO V_INSCMUOPER, V_CDINSCOPER, V_SERIEOPER
   FROM EMPRCNTRFIL
   WHERE CDEMPRESA = V_CDEMPROPER   AND
         CDFILIAL  = P_CDFILIAPUR;

   IF V_SERIEATAC IS NULL THEN
      V_SERIEATAC    := V_CDSERIE;
      V_INSCMUATAC   := V_CDINSCMUNI;
      V_CDINSCATAC   := V_CDINSCESTA;
      SELECT NVL(MAX(NRMAXITNF),10)
      INTO V_TTITENSNF
      FROM SERIENF
      WHERE CDEMPRESA  = V_CDEMPRESA  AND
            CDINSCESTA = V_CDINSCATAC  AND
            CDINSCMUNI = V_INSCMUATAC  AND
            CDSERIE    = V_SERIEATAC;
   ELSE
      SELECT NVL(MAX(NRMAXITNF),10)
      INTO V_TTITENSNF
      FROM SERIENF
      WHERE CDEMPRESA  = V_CDEMPRATAC  AND
            CDINSCESTA = V_CDINSCATAC  AND
            CDINSCMUNI = V_INSCMUATAC  AND
            CDSERIE    = V_SERIEATAC;
   END IF;

   IF V_SERIEOPER IS NULL THEN
      V_SERIEOPER    := V_CDSERIE;
      V_INSCMUOPER   := V_CDINSCMUNI;
      V_CDINSCOPER   := V_CDINSCESTA;
   END IF;

   /*DADOS DO EMITENTE*/
   SELECT NVL(MAX(IDENDCFOPFIL),'I'), MAX(CDEMPRESA), MAX(CDINSCESTA)
   INTO   V_IDENDCFOP,                V_CDEMPRESA,    V_CDINSCESTA
   FROM   FILIAL
   WHERE  (CDFILIAL = P_CDFILIAPUR);

   IF V_IDENDCFOP = 'I' THEN
      SELECT MAX(CDPAIS), MAX(SGESTADO), MAX(CDMUNICIPIO)
      INTO   V_PAISORIG,  V_ESTAORIG,    V_MUNIORIG
      FROM   INSCRESTAD
      WHERE  CDEMPRESA  = V_CDEMPRESA AND
             CDINSCESTA = V_CDINSCESTA;
   ELSE
      SELECT MAX(CDPAIS), MAX(SGESTADO), MAX(CDMUNICIPIO)
      INTO   V_PAISORIG,  V_ESTAORIG,    V_MUNIORIG
      FROM   ENDEFILI
      WHERE  (CDFILIAL     = P_CDFILIAPUR) AND
             (IDTPENDEFILI = V_IDENDCFOP);
   END IF;

   /*DADOS DO DESTINIATARIO */
   SELECT MAX(IDORIGDESTOP)
   INTO   V_ORIGDESTOP
   FROM   TIPOOPERAC
   WHERE  IDENTRSAIDOP = 'S' AND
          CDTIPOOPER   = '01';

   IF V_ORIGDESTOP = 'L' THEN /*QUANDO O DESTINATARIO  UMA FILIAL */
      SELECT NVL(MAX(IDENDEMISFIL),'I'), MAX(CDEMPRESA), MAX(CDINSCESTA)
      INTO   V_IDENDEMIS,                V_CDEMPRESADEST,    V_CDINSCESTADEST
      FROM   FILIAL
      WHERE  (CDFILIAL = P_CDCLIENTE);

      IF V_IDENDEMIS = 'I' THEN
         SELECT MAX(CDPAIS), MAX(SGESTADO), MAX(CDMUNICIPIO)
         INTO   V_PAISDEST,  V_ESTADEST,    V_MUNIDEST
         FROM   INSCRESTAD
         WHERE  CDEMPRESA  = V_CDEMPRESADEST AND
                CDINSCESTA = V_CDINSCESTADEST;
      ELSE
         SELECT MAX(CDPAIS), MAX(SGESTADO), MAX(CDMUNICIPIO)
         INTO   V_PAISDEST,  V_ESTADEST,    V_MUNIDEST
         FROM   ENDEFILI
         WHERE  CDFILIAL     = P_CDCLIENTE AND
                IDTPENDEFILI = V_IDENDEMIS;
      END IF;
   ELSIF V_ORIGDESTOP = 'F' THEN /*QUANDO O DESTINATARIO  UM FORNECEDOR*/
      SELECT NVL(MAX(IDENDEMISFOR),'P')
      INTO   V_IDENDEMIS
      FROM   FORNECEDOR
      WHERE  (CDFORNECED = P_CDCLIENTE);

      SELECT MAX(CDPAIS), MAX(SGESTADO), MAX(CDMUNICIPIO)
      INTO   V_PAISDEST,  V_ESTADEST,    V_MUNIDEST
      FROM   ENDEFORN
      WHERE  (CDFORNECED   = P_CDCLIENTE) AND
             (IDTPENDEFORN = V_IDENDEMIS);
   ELSE /*QUANDO O DESTINATARIO  UM CLIENTE*/
      SELECT NVL(MAX(IDENDEMISCLI),'P')
      INTO   V_IDENDEMIS
      FROM   CLIENTE
      WHERE  (CDCLIENTE = P_CDCLIENTE);

      SELECT MAX(CDPAIS), MAX(SGESTADO), MAX(CDMUNICIPIO)
      INTO   V_PAISDEST,  V_ESTADEST,    V_MUNIDEST
      FROM   ENDECLIE
      WHERE  (CDCLIENTE    = P_CDCLIENTE) AND
             (IDTPENDECLIE = V_IDENDEMIS);
   END IF;
   /*VERIFICANDO OS PARAMETROS INTERESTADUAL E INTERNACIONAL*/
   V_IDINTEREST := 'N';
   V_IDINTERNA  := 'N';
   IF ((V_PAISDEST = V_PAISORIG) AND (V_ESTADEST <> V_ESTAORIG)) THEN
      V_IDINTEREST := 'S';
   ELSIF (V_PAISDEST <> V_PAISORIG) THEN
      V_IDINTERNA := 'S';
   END IF;

   /* APAGA OS REGISTROS DA TRAB_APURAC_GD DE APURACOES ANTERIORES */
   DELETE FROM TRAB_APURAC_GD
   WHERE CDFILIALAPGD  = P_CDFILIPROC  AND
         NRAPURACGD    = P_NRAPURACFAT AND
         CDCLIENTEAPGD = P_CDCLIENTE;

   /* BUSCA OS PARAMETROS DA PARAMGERAL E PARAMFILIAL */
   SELECT MAX(SUBSTR(DSPARAMFATGE,11,1)) INTO stIdTipNFEmp
   FROM PARAMGERAL;

   IF (stIdTipNFEmp IS NULL) OR (RTRIM(stIdTipNFEmp) = '') THEN
      stIdTipNFEmp := '2';
      -- REFEIO
   END IF;

   SELECT MAX(SUBSTR(DSPARAMFATFI,01,1)), MAX(SUBSTR(DSPARAMFATFI,02,1)),
          MAX(SUBSTR(DSPARAMFATFI,15,1))
   INTO stIdMiniEfet, stIdQtdePrec, stIdUtEfePla
   FROM PARAMFILIAL
   WHERE CDFILIAL = P_CDFILIAPUR;

   IF (stIdMiniEfet IS NULL) OR (stIdMiniEfet = ' ') THEN
      stIdMiniEfet := '1';
      -- POR DIA
   END IF;
   IF (stIdQtdePrec IS NULL) OR (stIdQtdePrec = ' ') THEN
      stIdQtdePrec := '1';
      -- POR QUANTIDADE
   END IF;
   IF (stIdUtEfePla IS NULL) OR (stIdUtEfePla = ' ') THEN
      stIdUtEfePla := 'N';
      -- NAO UTILIZA EFETIVOS DO PLANEJAMENTO
   END IF;

   CONSULTA_GRUPOFISCAL('C', P_CDCLIENTE, P_CDFILIAPUR, 'S', '01', V_CDGRUPOFISC);

   /* ABRE O CURSOR */
   OpenGDiretaCursor(Cur_GDireta, stIdMiniEfet, stIdUtEfePla, 'N');
   LOOP
      FETCH Cur_GDireta INTO Rec_GDireta;

      EXIT WHEN Cur_GDireta%NOTFOUND;

      /* CALCULA O MINIMO DE EFETIVOS */
      QtdeMiniEfet := UTIL_APURAC.CalcEfetMini(Rec_GDireta.CdFilial,
                                               Rec_GDireta.CdCliente,
                                               Rec_GDireta.CdServFili,
                                               Rec_GDireta.CdGrServFat,
                                               Rec_GDireta.DtFaturam);

      V_VrServico  := Rec_GDireta.VrServico;
      V_QtEfetServ := Rec_GDireta.QtEfetivos;

      /* EFETUA O AJUSTE DAS QUANTIDADES OU VALORES DOS SERVICOS */
      IF (stIdMiniEfet = '1') THEN
      -- POR DIA
         IF (stIdQtdePrec = '1') THEN
         -- POR QUANTIDADE
            IF (Rec_GDireta.QtEfetivos < QtdeMiniEfet) THEN
               V_QtEfetServ := QtdeMiniEfet;
            END IF;
         ELSE
         -- POR PRECO
            UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                  Rec_GDireta.CdCliente,
                                  Rec_GDireta.CdServFili,
                                  Rec_GDireta.CdGrServFat,
                                  V_QtEfetServ,
                                  QtdeMiniEfet,
                                  'S',
                                  V_VrServico,
                                  V_VrPercCorr);
         END IF;
      ELSE
      -- POR PERIODO

         /* VERIFICA SE JA FOI GRAVADO REGISTRO PARA AJUSTE DE EFETIVOS
            POR QUANTIDADE */
         SELECT COUNT(*)
         INTO V_Existe
         FROM TRAB_APURAC_GD
         WHERE CDFILIALAPGD   = P_CDFILIPROC  AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = Rec_GDireta.CdCliente AND
               CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
               IDAJUSEFETAPGD = 'S';

         IF (V_Existe = 0) THEN
            /* CALCULA O TOTAL DE EFETIVOS DO PERIODO */
            SELECT SUM(QTEFETIVOS)
            INTO V_TtEfetServ
            FROM SUM_EFETIVOS
            WHERE (DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM) AND
                  ((IDFATUPLAN = 'F') OR (stIdUtEfePla = 'S')) AND
                  CDFILIAL   = Rec_GDireta.CdFilial AND
                  CDCLIENTE  = Rec_GDireta.CdCliente AND
                  CDSERVFILI = Rec_GDireta.CdServFili;

            IF (stIdQtdePrec = '1') THEN
            -- POR QUANTIDADE
               IF (V_TtEfetServ < QtdeMiniEfet) THEN
                  SELECT MAX(VRSERVFATU)
                  INTO V_VrServFatu
                  FROM SERVCLIE
                  WHERE CDFILIAL   = Rec_GDireta.CdFilial AND
                        CDCLIENTE  = Rec_GDireta.CdCliente AND
                        CDSERVFILI = Rec_GDireta.CdServFili;

                  IF (V_VrServFatu IS NULL) THEN
                     V_VrServFatu := 0;
                  END IF;

                  CONSULTA_CST('S',          V_CdEmpresa,   Rec_GDireta.CdProduto, V_CdInscEsta, '01',
                               P_CDFILIPROC, P_NRAPURACFAT, V_CDGRUPOFISC,         V_IDINTEREST, V_IDINTERNA);

                  SELECT NVL(MAX(CDSITTRIBTMP),'00')
                  INTO V_CSTTEMP
                  FROM CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;

                  IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
                     V_IDPRODSUBS := 'S';
                  ELSE
                     V_IDPRODSUBS := 'N';
                  END IF;

                  /* GRAVA O AJUSTE DA QUANTIDADE DE EFETIVOS */
                  INSERT INTO TRAB_APURAC_GD
                  (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                   CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
                   NREFETIVOSAPGD, VRPRECUNITAPGD,
                   IDAJUSEFETAPGD, IDPRODSUBSGD)
                  VALUES (P_CDFILIPROC, P_NRAPURACFAT, Rec_GDireta.CdCliente,
                          Rec_GDireta.CdServFili, P_DATAINI, Rec_GDireta.CdProduto,
                          QtdeMiniEfet - V_TtEfetServ, V_VrServFatu,
                          'S', V_IDPRODSUBS);

                  DELETE CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;
               END IF;
            ELSE
            -- POR PRECO
               IF (V_TtEfetServ < QtdeMiniEfet) THEN
                  UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                        Rec_GDireta.CdCliente,
                                        Rec_GDireta.CdServFili,
                                        Rec_GDireta.CdGrServFat,
                                        V_TtEfetServ,
                                        QtdeMiniEfet,
                                        'S',
                                        V_VrServico,
                                        V_VrPercCorr);
               END IF;
            END IF;
         END IF;
      END IF;

      /* GRAVA OS SERVICOS NA TRAB_APURAC_GD JA COM AS QUANTIDADES
         E VALORES CORRIGIDOS */
      SELECT COUNT(*)
      INTO V_Existe
      FROM TRAB_APURAC_GD
      WHERE CDFILIALAPGD   = P_CDFILIPROC  AND
            NRAPURACGD     = P_NRAPURACFAT AND
            CDCLIENTEAPGD  = Rec_GDireta.CdCliente AND
            CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
            VRPRECUNITAPGD = ROUND(V_VrServico,2);

      IF (V_Existe = 0) THEN
         CONSULTA_CST('S',          V_CdEmpresa,   Rec_GDireta.CdProduto, V_CdInscEsta, '01',
                      P_CDFILIPROC, P_NRAPURACFAT, V_CDGRUPOFISC,         V_IDINTERNA,  V_IDINTERNA);

         SELECT NVL(MAX(CDSITTRIBTMP),'00')
         INTO V_CSTTEMP
         FROM CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;

         IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
            V_IDPRODSUBS := 'S';
         ELSE
            V_IDPRODSUBS := 'N';
         END IF;

         INSERT INTO TRAB_APURAC_GD
         (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
          CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
          NREFETIVOSAPGD, VRPRECUNITAPGD,
          IDAJUSEFETAPGD, IDPRODSUBSGD)
         VALUES (P_CDFILIPROC, P_NRAPURACFAT, Rec_GDireta.CdCliente,
                 Rec_GDireta.CdServFili, P_DATAINI, Rec_GDireta.CdProduto,
                 V_QtEfetServ, ROUND(V_VrServico,2),
                 'N', V_IDPRODSUBS);

         DELETE CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;
      ELSE
         UPDATE TRAB_APURAC_GD
         SET NREFETIVOSAPGD = NREFETIVOSAPGD + V_QtEfetServ
         WHERE CDFILIALAPGD   = P_CDFILIPROC  AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = Rec_GDireta.CdCliente AND
               CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
               VRPRECUNITAPGD = ROUND(V_VrServico,2);
      END IF;
   END LOOP;
   CLOSE Cur_GDireta;

   /* ABRE O CURSOR PARA TODOS OS SERVICOS QUE POSSUEM GRUPOS */
   OpenGDiretaCursor(Cur_GDireta, stIdMiniEfet, stIdUtEfePla, 'S');
   LOOP
      FETCH Cur_GDireta INTO Rec_GDireta;
      EXIT WHEN Cur_GDireta%NOTFOUND;

      /* CALCULA O MINIMO DE EFETIVOS */
      QtdeMiniEfet := UTIL_APURAC.CalcEfetMini(Rec_GDireta.CdFilial,
                                               Rec_GDireta.CdCliente,
                                               Rec_GDireta.CdServFili,
                                               Rec_GDireta.CdGrServFat,
                                               Rec_GDireta.DtFaturam);

      V_VrServico  := Rec_GDireta.VrServico;
      V_QtEfetServ := Rec_GDireta.QtEfetivos;

      SELECT SUM(QTEFETIVOS)
      INTO V_QtEfetGrup
      FROM SUM_EFETIVOS
      WHERE (DTFATURAM = Rec_GDireta.DtFaturam) AND
            ((IDFATUPLAN = 'F') OR (stIdUtEfePla = 'S')) AND
            CDFILIAL    = Rec_GDireta.CdFilial AND
            CDCLIENTE   = Rec_GDireta.CdCliente AND
            CDGRSERVFAT = Rec_GDireta.CdGrServFat;

      SELECT MAX(CDSERAJEFEGSF)
      INTO V_CdServAjus
      FROM GRUPSERVFAT
      WHERE CDFILIAL    = Rec_GDireta.CdFilial AND
            CDCLIENTE   = Rec_GDireta.CdCliente AND
            CDGRSERVFAT = Rec_GDireta.CdGrServFat;

      /* EFETUA O AJUSTE DAS QUANTIDADES OU VALORES DOS SERVICOS */
      IF (stIdMiniEfet = '1') THEN
      -- POR DIA
         IF (stIdQtdePrec = '1') THEN
         -- POR QUANTIDADE
            IF (V_QtEfetGrup < QtdeMiniEfet) AND (Rec_GDireta.CdServFili = V_CdServAjus) THEN
               V_QtEfetServ := V_QtEfetServ + (QtdeMiniEfet - V_QtEfetGrup);
            END IF;
         ELSE
         -- POR PRECO
            UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                  Rec_GDireta.CdCliente,
                                  Rec_GDireta.CdServFili,
                                  Rec_GDireta.CdGrServFat,
                                  V_QtEfetGrup,
                                  QtdeMiniEfet,
                                  'G',
                                  V_VrServico,
                                  V_VrPercCorr);
         END IF;
      ELSE
      -- POR PERIODO

         /* VERIFICA SE JA FOI GRAVADO REGISTRO PARA AJUSTE DE EFETIVOS
            POR QUANTIDADE */
         SELECT COUNT(*)
         INTO V_Existe
         FROM TRAB_APURAC_GD
         WHERE CDFILIALAPGD   = P_CDFILIPROC  AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = Rec_GDireta.CdCliente AND
               CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
               IDAJUSEFETAPGD = 'S';

         IF (V_Existe = 0) THEN
            /* CALCULA O TOTAL DE EFETIVOS DO PERIODO */
            SELECT SUM(QTEFETIVOS)
            INTO V_QtEfetGrup
            FROM SUM_EFETIVOS
            WHERE (DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM) AND
                  ((IDFATUPLAN = 'F') OR (stIdUtEfePla = 'S')) AND
                  CDFILIAL    = Rec_GDireta.CdFilial AND
                  CDCLIENTE   = Rec_GDireta.CdCliente AND
                  CDGRSERVFAT = Rec_GDireta.CdGrServFat;

            IF (stIdQtdePrec = '1') THEN
            -- POR QUANTIDADE
               IF (V_QtEfetGrup < QtdeMiniEfet) AND (Rec_GDireta.CdServFili = V_CdServAjus) THEN
                  SELECT MAX(VRSERVFATU)
                  INTO V_VrServFatu
                  FROM SERVCLIE
                  WHERE CDFILIAL   = Rec_GDireta.CdFilial AND
                        CDCLIENTE  = Rec_GDireta.CdCliente AND
                        CDSERVFILI = Rec_GDireta.CdServFili;

                  IF (V_VrServFatu IS NULL) THEN
                     V_VrServFatu := 0;
                  END IF;

                  CONSULTA_CST('S',          V_CdEmpresa,   Rec_GDireta.CdProduto, V_CdInscEsta, '01',
                               P_CDFILIPROC, P_NRAPURACFAT, V_CDGRUPOFISC,         V_IDINTERNA,  V_IDINTERNA);

                  SELECT NVL(MAX(CDSITTRIBTMP),'00')
                  INTO V_CSTTEMP
                  FROM CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;

                  IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
                     V_IDPRODSUBS := 'S';
                  ELSE
                     V_IDPRODSUBS := 'N';
                  END IF;

                  /* GRAVA O AJUSTE DA QUANTIDADE DE EFETIVOS */
                  INSERT INTO TRAB_APURAC_GD
                  (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                   CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
                   NREFETIVOSAPGD, VRPRECUNITAPGD,
                   IDAJUSEFETAPGD, IDPRODSUBSGD)
                  VALUES (P_CDFILIPROC, P_NRAPURACFAT, Rec_GDireta.CdCliente,
                          Rec_GDireta.CdServFili, P_DATAINI, Rec_GDireta.CdProduto,
                          QtdeMiniEfet - V_QtEfetGrup, V_VrServFatu,
                          'S', V_IDPRODSUBS);

                  DELETE CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;
               END IF;
            ELSE
            -- POR PRECO
               IF (V_QtEfetGrup < QtdeMiniEfet) THEN
                  UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                        Rec_GDireta.CdCliente,
                                        Rec_GDireta.CdServFili,
                                        Rec_GDireta.CdGrServFat,
                                        V_QtEfetGrup,
                                        QtdeMiniEfet,
                                        'G',
                                        V_VrServico,
                                        V_VrPercCorr);
               END IF;
            END IF;
         END IF;
      END IF;

      /* GRAVA OS SERVICOS NA TRAB_APURAC_GD JA COM AS QUANTIDADES
         E VALORES CORRIGIDOS */
      SELECT COUNT(*)
      INTO V_Existe
      FROM TRAB_APURAC_GD
      WHERE CDFILIALAPGD   = P_CDFILIPROC  AND
            NRAPURACGD     = P_NRAPURACFAT AND
            CDCLIENTEAPGD  = Rec_GDireta.CdCliente AND
            CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
            VRPRECUNITAPGD = ROUND(V_VrServico,2);

      IF (V_Existe = 0) THEN
         CONSULTA_CST('S',          V_CdEmpresa,   Rec_GDireta.CdProduto, V_CdInscEsta, '01',
                      P_CDFILIPROC, P_NRAPURACFAT, V_CDGRUPOFISC,         V_IDINTERNA,  V_IDINTERNA);

         SELECT NVL(MAX(CDSITTRIBTMP),'00')
         INTO V_CSTTEMP
         FROM CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;

         IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
            V_IDPRODSUBS := 'S';
         ELSE
            V_IDPRODSUBS := 'N';
         END IF;

         INSERT INTO TRAB_APURAC_GD
         (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
          CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
          NREFETIVOSAPGD, VRPRECUNITAPGD,
          IDAJUSEFETAPGD, IDPRODSUBSGD)
         VALUES (P_CDFILIPROC, P_NRAPURACFAT, Rec_GDireta.CdCliente,
                 Rec_GDireta.CdServFili, P_DATAINI, Rec_GDireta.CdProduto,
                 V_QtEfetServ, ROUND(V_VrServico,2),
                 'N', V_IDPRODSUBS);

         DELETE CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;
      ELSE
         UPDATE TRAB_APURAC_GD
         SET NREFETIVOSAPGD = NREFETIVOSAPGD + V_QtEfetServ
         WHERE CDFILIALAPGD   = P_CDFILIPROC  AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = Rec_GDireta.CdCliente AND
               CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
               VRPRECUNITAPGD = ROUND(V_VrServico,2);
      END IF;
   END LOOP;
   CLOSE Cur_GDireta;

   /* APAGA TODAS AS NOTAS FISCAIS APURADAS ANTERIORMENTE COM MESMO
   NUMERO DE APURACAO */
   FOR CURS_NF IN (SELECT CDFILIAL, NRLANCTONF
                   FROM NOTAFISCAL
                   WHERE (CDFILIAL    = P_CDFILIPROC) AND
                         (NRAPURACFAT = P_NRAPURACFAT)) LOOP
      DELETA_NF(CURS_NF.CDFILIAL, CURS_NF.NRLANCTONF);
   END LOOP;

   V_NRLANCTONF := P_NRLANCTONF;

   SELECT SUM(NVL(NREFETIVOSAPGD,0) * NVL(VRPRECUNITAPGD,0))
   INTO V_VRTOTASERV
   FROM TRAB_APURAC_GD
   WHERE CDFILIALAPGD   = P_CDFILIPROC  AND
         NRAPURACGD     = P_NRAPURACFAT;
         
   /*BUSCA DOS TIPOS DE OPERAO SELECIONADOS PARA REFEIO E PRESTAO DE SERVIOS,*/
   /*RESPECTIVAMENTE, CADASTRADOS NA TELA "PARMETROS POR TIPO DE FATURAMENTO"*/
   SELECT 
      NVL(CDTPOPERREF, '01'),
      NVL(CDTPOPERPRESER, '01')
   INTO
      V_CDTPOPERREF,
      V_CDTPOPERPRESER
   FROM 
      PARAMGERAL; 

   IF V_VRTOTASERV > 0 THEN
     /*Gerao de Nota Fiscal de Refeio*/

      V_VRLIQUFATU := (V_VRTOTASERV * ((100 - V_VRPERCSERV) / 100));

      V_CONTITEM  := V_TTITENSNF;

      IF V_VRLIQUFATU > 0 THEN
         FOR CURS_APURAC IN (SELECT CDFILIALAPGD, CDCLIENTEAPGD,
                                 CDSERVFILIAPGD, CDPRODUTOAPGD,
                                 NREFETIVOSAPGD, IDAJUSEFETAPGD, NMSERVICO, IDPRODSUBSGD,
                                 ROUND((VRPRECUNITAPGD * ((100 - V_VRPERCSERV) / 100)),2) AS VRPRECUNITAPGD
                             FROM TRAB_APURAC_GD, SERVICO
                             WHERE (TRAB_APURAC_GD.CDFILIALAPGD   = P_CDFILIPROC)       AND
                                   (TRAB_APURAC_GD.NRAPURACGD     = P_NRAPURACFAT)      AND
                                   (TRAB_APURAC_GD.CDSERVFILIAPGD = SERVICO.CDSERVFILI) AND
                                   (SERVICO.CDFILIAL              = P_CDFILIAPUR)
                             ORDER BY CDFILIALAPGD, CDCLIENTEAPGD, IDPRODSUBSGD, CDSERVFILIAPGD) LOOP

           V_VRNFATACA  := NVL(V_VRNFATACA,0) + (CURS_APURAC.VRPRECUNITAPGD * CURS_APURAC.NREFETIVOSAPGD);

           IF (V_CONTITEM = V_TTITENSNF) THEN
               /* EFETUA O FECHAMENTO DA NOTA, GERACAO DE IMPOSTOS */

               V_CONTITEM   := 0;

               NOVO_CODIGO('LANCNF/FILIAL' || P_CDFILIPROC,V_NRLANCTONF);
               V_NRLANCTONF := SUBSTR(V_NRLANCTONF,11,10);

               GERA_NOTAFISCAL_CORPO(P_CDFILIPROC,  P_CDFILIAPUR, V_CDEMPRATAC,
                                     V_CDTPOPERREF, V_SERIEATAC,  P_CDCLIENTE,
                                     P_DATAFIM,     'FAI',        V_NRLANCTONF,
                                     V_NRLANCESTQ,  'N',          P_NRAPURACFAT,
                                     V_NRLANCTONF,   NULL,        NULL,
                                     NULL,           NULL,        NULL,
                                     NULL,           NULL,        NULL,
                                     NULL,           NULL,        NULL,
                                     NULL,           NULL,        NULL,
                                     NULL,           P_DATAFIM);

            END IF;
            /*Incluso dos itens da nota fiscal*/
            V_CONTITEM := V_CONTITEM + 1;

            GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_NRLANCTONF, CURS_APURAC.CDPRODUTOAPGD,
                                    CURS_APURAC.CDSERVFILIAPGD, CURS_APURAC.NMSERVICO,
                                    CURS_APURAC.NREFETIVOSAPGD, CURS_APURAC.VRPRECUNITAPGD);

            IF (V_CONTITEM = V_TTITENSNF) THEN
               GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_NRLANCTONF);
            END IF;

         END LOOP;

         /*Efetua o fechamento da nota, geracao de impostos*/
         GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_NRLANCTONF);
      END IF;

      /*Gerao de Nota Fiscal de Servio*/

      NOVO_CODIGO('LANCNF/FILIAL' || P_CDFILIPROC, V_NRLANCTONF);
      V_NRLANCTONF := SUBSTR(V_NRLANCTONF,11,10);
      V_VRLIQUFATU := (V_VRTOTASERV - V_VRNFATACA);

      IF V_VRLIQUFATU > 0 THEN

         GERA_NOTAFISCAL_CORPO(P_CDFILIPROC,     P_CDFILIAPUR, V_CDEMPROPER,
                               V_CDTPOPERPRESER, V_SERIEOPER,  P_CDCLIENTE,
                               P_DATAFIM,        'FAI',        V_NRLANCTONF,
                               V_NRLANCESTQ,     'N',          P_NRAPURACFAT,
                               V_NRLANCTONF,     NULL,        NULL,
                               NULL,             NULL,        NULL,
                               NULL,             NULL,        NULL,
                               NULL,             NULL,        NULL,
                               NULL,             NULL,        NULL,
                               NULL,             P_DATAFIM);

         /*Incluso dos itens da nota fiscal*/
         GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_NRLANCTONF, V_CDPRGENEREF,
                                 V_CDPRGENEREF, V_NMPRGENEREF, 1, ROUND(V_VRLIQUFATU,2), 0,
                                 NULL, NULL, NULL, NULL,
                                 NULL, NULL);

         /*Efetua o fechamento da nota, geracao de impostos*/
         GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_NRLANCTONF);
      END IF;

      DELETE FROM TRAB_APURAC_GD
      WHERE CDFILIALAPGD  = P_CDFILIPROC    AND
            NRAPURACGD    = P_NRAPURACFAT   AND
            CDCLIENTEAPGD = P_CDCLIENTE;
   END IF;
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('APURAC_REFESERV','5.30.00'); END;
/

